#include "automata.h"

#include <cctype>
#include <algorithm>
#include <stdexcept>
#include <string>

namespace lxs {
    std::string toDot(const DFA& d)
    {
        std::string s = "digraph {\nrankdir=LR\nin [shape=point style=invis]";

        for (State state = 0; state < d.numStates; state++)
        {
            s += std::to_string(state) + " [label=\"" + std::to_string(state) + "\"";
            if (state == d.starting)
                s += " color=yellow";
            if (d.accepting.count(state) > 0)
                s += " color=green shape=doublecircle";
            s += "]\n";
        }

        for (const auto& tmp : d.delta)
        {
            const auto& from = tmp.first;
            for (const auto& trans : tmp.second)
            {
                s += std::to_string(from) + " -> " + std::to_string(trans.second) + " [label=\"" + trans.first + "\"]\n";
            }
        }

        s += "in -> " + std::to_string(d.starting) + "\n}\n";
        return s;
    }

    std::string toDot(const NFA& n)
    {
        std::string s = "digraph {\nrankdir=LR\nin [shape=point style=invis]\n";

        for (State state = 0; state < n.numStates; state++)
        {
            s += std::to_string(state) + " [label=\"" + std::to_string(state) + "\"";
            if (state == n.starting)
                s += " color=yellow";
            if (n.accepting.count(state) > 0)
                s += " color=green shape=doublecircle";
            s += "]\n";
        }

        for (const auto& tmp : n.delta)
        {
            const auto& from = tmp.first;
            for (const auto& trans : tmp.second)
            {
                for (const auto& to : trans.second)
                    s += std::to_string(from) + " -> " + std::to_string(to) + " [label=\"" + trans.first + "\"]\n";
            }
        }

        s += "in -> " + std::to_string(n.starting) + "\n}\n";
        return s;
    }

    std::string toDot(const ENFA& e)
    {
        std::string s = "digraph {\nrankdir=LR\nin [shape=point style=invis]\n";

        for (State state = 0; state < e.numStates; state++)
        {
            s += std::to_string(state) + " [label=\"" + std::to_string(state) + "\"";
            if (state == e.starting)
                s += " color=yellow";
            if (e.accepting.count(state) > 0)
                s += " color=green shape=doublecircle";
            s += "]\n";
        }

        for (const auto& tmp : e.delta)
        {
            const auto& from = tmp.first;
            for (const auto& trans : tmp.second)
            {
                for (const auto& to : trans.second)
                    s += std::to_string(from) + " -> " + std::to_string(to) + " [label=\"" + trans.first + "\"]\n";
            }
        }

        for (const auto& etrans : e.epsilonTransitions)
        {
            for (const auto& dest : etrans.second)
                s += std::to_string(etrans.first) + " -> " + std::to_string(dest) + " [label=\"ε\"]\n";
        }

        s += "in -> " + std::to_string(e.starting) + "\n}\n";
        return s;
    }
} //namespace lxs
